@@ -11,7 +11,6 @@ from typing import (
     Sequence,
     Tuple,
     Union,
-    cast,
 )
 
 from django.contrib.admin.widgets import AutocompleteMixin
@@ -184,24 +183,20 @@ class AutocompleteSelect(Select):
                 if v.startswith("{") and v.endswith("}"):
                     parsed_v = json.loads(v)
                 else:
-                    parsed_v = {"text": v, "value": v, "extra": {}}
-
-            # Ensure that the selected option has a JSON-serialized
-            # representation of itself in its "id" field.
-            parsed_v.pop("id", None)
-            parsed_v["id"] = stable_json(parsed_v)
+                    freetext_option = {"text": v, "value": v, "extra": {}}
+                    parsed_v = {**freetext_option, "id": stable_json(freetext_option)}
 
             if parsed_v is not None:
                 parsed_value.append(parsed_v)
 
         # If only one value is expected, unwrap it from the parsed list.
         final_value = (
-            next(iter(cast(Iterable, parsed_value)), None)
+            next(iter(parsed_value), None)
             if not self.allow_multiple_selected
             else parsed_value
         )
 
-        return json.dumps(final_value)
+        return stable_json(final_value)
 
     def format_value(self, value: Any) -> Any:
         """Format a value for rendering in a widget template.
